@@ -4,15 +4,138 @@ from logging import handlers
 import logging
 from threading import Thread
 import time
+import sqlite3
 from pycqBot.socketApp import cqSocket, asyncHttp
 from pycqBot.cqCode import strToCqCodeToDict
 
 
 class cqHttpApi(asyncHttp):
 
-    def __init__(self, ip="127.0.0.1",port=8000, download_path="./download", chunk_size=1024) -> None:
+    def __init__(self, host="http://127.0.0.1:8000", download_path="./download", chunk_size=1024) -> None:
         super().__init__(download_path, chunk_size)
-        self.http = "http://%s:%s" % (ip, port)
+        self.http = host
+        self.__reply_list_msg = {}
+        self.thread_count = 4
+        self.bot_qq = 0
+
+    def create_bot(self, host="ws://127.0.0.1:5700", group_id_list=[], user_id_list=[], options={}):
+        """
+        直接创建一个 bot 
+        """
+        return cqBot(
+            self, host, group_id_list, user_id_list, options
+        )
+    
+    def _create_sql_link(self, db_path, sleep):
+        """
+        长效消息存储 初始化
+        """
+        db_path = os.path.join(db_path, "bot_sql.db")
+        self._db_path = db_path
+
+        if os.path.isfile(db_path):
+            os.remove(db_path)
+
+        with sqlite3.connect(self._db_path) as sql_link:
+            sql_cursor = sql_link.cursor()
+            # 初始化 Message 表
+            sql_cursor.execute("""CREATE TABLE `Message` (
+                    ID               INTEGER PRIMARY KEY AUTOINCREMENT,
+                    userId           NOT NULL,
+                    stime            NOT NULL,
+                    etime            NOT NULL,
+                    messageData JSON NOT NULL
+                );
+            """)
+
+            sql_link.commit()
+        
+        thread = Thread(target=self._record_message_ck, args=(sleep,),name="_record_message_ck")
+        thread.setDaemon(True)
+        thread.start()
+    
+    def _record_message_ck(self, sleep):
+        """
+        长效消息存储 检查失效消息
+        """
+        while True:
+            try:
+                with sqlite3.connect(self._db_path) as sql_link:
+                    sql_cursor = sql_link.cursor()
+                    data_list = sql_cursor.execute("SELECT * FROM `Message` WHERE etime < '%s'" % int(time.time()))
+                    for data in data_list:
+                        sql_cursor.execute("DELETE from `Message` where ID = '%s'" % data[0])
+                        self.recordMessageInvalid(data, sql_link)
+
+            except Exception as err:
+                self.recordMessageCKError(err)
+
+            time.sleep(sleep)
+
+    def record_message(self, message_data, time_end):
+        """
+        长效消息存储
+        """
+        time_int = int(time.time())
+        time_end = time_int + time_end
+        try:
+            with sqlite3.connect(self._db_path) as sql_link:
+                sql_cursor = sql_link.cursor()
+                sql_cursor.execute("""
+                    INSERT INTO `Message` VALUES (
+                        NULL , "%s", "%s", "%s", "%s" 
+                    )
+                """ % (message_data["user_id"], time_int, time_end, message_data))
+                sql_link.commit()
+        except Exception as err:
+            self.recordMessageError(message_data, time_int, time_end, err)
+    
+    def record_message_get(self, user_id):
+        """
+        长效消息存储 获取
+        """
+        try:
+            with sqlite3.connect(self._db_path) as sql_link:
+                sql_cursor = sql_link.cursor()
+                data_list = sql_cursor.execute("SELECT * FROM `Message` WHERE userId = '%s'" % user_id)
+                return data_list.fetchall()
+
+        except Exception as err:
+            self.recordMessageGetError(user_id, err)
+
+    def reply(self, user_id, sleep):
+        """
+        等待回复
+        """
+        in_time =  time.time()
+        sleep += in_time
+        self.__reply_list_msg[user_id] = {}
+        while in_time < sleep:
+            in_time = time.time()
+            if self.__reply_list_msg[user_id] == {}:
+                continue
+            
+            break
+        
+        reply_msg = self.__reply_list_msg[user_id]
+        self.__reply_list_msg.pop(user_id)
+
+        return reply_msg
+    
+    def _reply_ck(self, user_id):
+        """
+        等待回复 检查
+        """
+        if user_id in self.__reply_list_msg:
+            return True
+        
+        return False
+    
+    def _reply_add(self, user_id, msg):
+        """
+        等待回复 添加回复数据
+        """
+        self.__reply_list_msg[user_id] = msg
 
     def _link(self, api, data={}):
         try:
@@ -49,6 +172,16 @@ class cqHttpApi(asyncHttp):
             "auto_escape":auto_escape
         }
         self.add("/send_msg", post_data)
+    
+    def send_group_forward_msg(self, group_id, message):
+        """
+        发送合并转发 ( 群 )
+        """
+        post_data = {
+            "group_id":group_id,
+            "messages": message,
+        }
+        self.add("/send_group_forward_msg", post_data)
 
     def send_reply(self, from_message, message, auto_escape=False):
         """
@@ -213,27 +346,69 @@ class cqHttpApi(asyncHttp):
         }
         return self._link("/get_image", post_data)
     
-    def clean_cache(self):
-        self.add("/send_msg")
-    
-    def set_restart(self, delay=600):
+    async def _cqhttp_download_file(self, url, headers):
+        """
+        go-cqhttp 的内置下载 (异步)
+        """
         post_data = {
-            "delay": delay
+            "url":url,
+            "headers": headers,
+            "thread_count": self.thread_count
         }
-        self.add("/send_msg", post_data)
+        return (await self._asynclink("/download_file", post_data))["data"]["file"]
+    
+    def cqhttp_download_file(self, url, headers):
+        """
+        go-cqhttp 的内置下载
+        """
+        post_data = {
+            "url":url,
+            "headers": headers,
+            "thread_count": self.thread_count
+        }
+        return self._link("/download_file", post_data)["file"]
+    
+    def get_status(self):
+        """
+        获取 go-cqhttp 状态
+        """
+        return self._link("/get_status")
+    
+    def recordMessageInvalid(self, record_message_data, sql_link):
+        """
+        长效消息存储 消息失效
+        """
+        logging.debug("长效消息存储 消息失效 %s" % str(record_message_data))
+    
+    def recordMessageError(self, message_data, time_int, time_end, err):
+        """
+        长效消息存储 消息存储失败
+        """
+        logging.error("长效消息存储 消息存储失败 %s Error: %s" % (message_data, err))
+        logging.exception(err)
+    
+    def recordMessageGetError(self, user_id, err):
+        """
+        长效消息存储 消息查询失败
+        """
+        logging.error("长效消息存储 消息查询失败 user_id: %s Error: %s" % (user_id, err))
+        logging.exception(err)
+    
+    def recordMessageCKError(self, err):
+        """
+        长效消息存储 检查消息失败
+        """
+        logging.error("长效消息存储 检查消息失败 Error: %s" % err)
+        logging.exception(err)
+
 
 class cqBot(cqSocket):
     """
     cqBot 机器人
     """
 
-    def __init__(self, cqapi, on_group_msg=None, on_private_msg=None,
-            group_id_list=[], user_id_list=[],
-            command={},
-            timing={},
-            options={}
-        ):
-        super().__init__()
+    def __init__(self, cqapi, host, group_id_list=[], user_id_list=[], options={}):
+        super().__init__(host)
 
         self.cqapi = cqapi
         # bot qq
@@ -242,101 +417,193 @@ class cqBot(cqSocket):
         self.__group_id_list = group_id_list
         # 需处理私信
         self.__user_id_list = user_id_list
-        # 群信息勾子
-        self.on_group_msg = on_group_msg
-        # 群信息勾子
-        self.on_private_msg = on_private_msg
         # 指令列表
-        self.commandList = command
+        self._commandList = {}
         # 定时任务
-        self.timingList = timing
+        self._timingList = {}
         # 管理员列表
         self.admin = []
         # 指令标志符
-        self.commandSign = "/"
+        self.commandSign = "#"
         # 帮助信息模版
-        self.help_text = "本bot帮助信息!\n{help_command_text}\npycqbot v0.1.0"
-        # 自动启动连接
-        self.auto_start = True
-        # 自动启动定时任务
-        self.auto_timing_start = True
+        self.help_text_format = "本bot帮助信息!\n{help_command_text}\npycqbot v0.1.0"
+        self.help_command_text = ""
+        # 长效消息存储
+        self.messageSql = False
+        # 长效消息存储 数据库目录
+        self.messageSqlPath = "./"
+        # 长效消息存储 清理间隔
+        self.messageSqlClearTime = 60
+        # go_cqhttp 状态 通过心跳更新
+        self._go_cqhttp_status = {}
 
         for key in options.keys():
             if type(options[key]) is str:
                 exec("self.%s = '%s'" % (key, options[key]))
             else:
                 exec("self.%s = %s" % (key, options[key]))
-        
-        self.commandList["help"] = {
-            "type": "all",
-            "help": [
-                self.commandSign + "help - 显示本条帮助信息",
-            ]
-        }
 
-        self._check_set_command()
-        self._set_help_text()
+        """
+        内置指令 help
+            显示帮助信息
+        """
 
         def print_help(_, __, message, ___):
             self.cqapi.send_reply(message, self.help_text)
 
-        self.commandList["help"]["function"] = print_help
+        self.command(print_help, "help", {
+            "type": "all",
+            "help": [
+                self.commandSign + "help - 显示本条帮助信息",
+            ]
+        })
 
-        if self.auto_start:
-            self.link()
+        """
+        内置指令 status
+            查看 go-cqhttp 状态
+        """
+        
+        def status(_, __, message, ___):
+            if self._go_cqhttp_status == {}:
+                self.cqapi.send_reply(message, "go-cqhttp 心跳未被正常配置，请检查")
+                logging.warning("go-cqhttp 心跳未被正常配置")
+                return
+
+            status_msg = "bot (qq=%s) 是否在线：%s\n收到数据包：%s\n发送数据包：%s\n丢失数据包：%s\n接受信息：%s\n发送信息：%s\nTCP 链接断开：%s\n账号掉线次数：%s\n最后消息时间：%s" % (
+                self.__bot_qq,
+                self._go_cqhttp_status["online"],
+                self._go_cqhttp_status["stat"]["PacketReceived"],
+                self._go_cqhttp_status["stat"]["PacketSent"],
+                self._go_cqhttp_status["stat"]["PacketLost"],
+                self._go_cqhttp_status["stat"]["MessageReceived"],
+                self._go_cqhttp_status["stat"]["MessageSent"],
+                self._go_cqhttp_status["stat"]["DisconnectTimes"],
+                self._go_cqhttp_status["stat"]["LostTimes"],
+                self._go_cqhttp_status["stat"]["LastMessageTime"],
+            )
+
+            self.cqapi.send_reply(message, status_msg)
+        
+        self.command(status, "status", {
+            "type": "all",
+            "admin": True,
+            "help": [
+                self.commandSign + "status - 获取 go-cqhttp 状态",
+            ]
+        })
+        
+    def _check_command_options(self, options):
+        """
+        检查指令设置
+        """
+        if "type" not in options:
+            options["type"] = "group"
+        
+        if "admin" not in options:
+            options["admin"] = False
+        
+        if "user" not in options:
+            options["user"] = ["all"]
+        else:
+            options["user"] = options["user"].split(",")
+        
+        if "ban" not in options:
+            options["ban"] = []
+
+        if "help" in options:
+            for help_r_text in options["help"]:
+                self.help_command_text = "%s%s\n" % (self.help_command_text, help_r_text)
+            
+            self.help_text = "%s\n" % self.help_text_format.format(help_command_text=self.help_command_text)
+        
+        return options
+    
+    def _check_timing_options(self, options, timing_name):
+        options["name"] = timing_name
+
+        if "timeSleep" not in options:
+            logging.warning("定时任务 %s 没有指定 timeSleep 间隔, 中止创建" % timing_name)
+            return False
+
+        if "ban" not in options:
+            options["ban"] = []
+
+        return options
+        
+    def command(self, function, command_name, options=None):
+        if options is None:
+            options = {}
+            
+        options = self._check_command_options(options)
+
+        if type(command_name) == str:
+            command_name = [command_name]
+
+        for name in command_name:
+            self._commandList[name] = options
+            self._commandList[name]["function"] = function
+
+    def _timing_job(self, job):
+        run_count = 0
+        while True:
+            self.timing_jobs_start(job, run_count)
+            for group_id in self.__group_id_list:
+                if group_id in job["ban"]:
+                    return
+                
+                run_count += 1
+                try:
+                    job["function"](group_id)
+                    self.timing_job_end(job, run_count, group_id)
+
+                except Exception as err:
+                    self.runTimingError(job, run_count, err, group_id)
+
+            self.timing_jobs_end(job, run_count)
+            time.sleep(job["timeSleep"])
+    
+    def timing(self, function, timing_name, options=None):
+        if options is None:
+            options = {}
+
+        options = self._check_timing_options(options, timing_name)
+        if not options:
+            return
+
+        options["function"] = function
+        self._timingList[timing_name] = options
+
+        thread = Thread(target=self._timing_job, args=(self._timingList[timing_name],), name=timing_name)
+        thread.setDaemon(True)
+        thread.start()
+
+        logging.info("创建定时任务 %s " % timing_name)
+    
+    def set_bot_status(self, message):
+        self.__bot_qq = message["self_id"]
+        self.cqapi.bot_qq = message["self_id"]
     
     def _meta_event_connect(self, message):
         """
         连接响应
         """
-        self.__bot_qq = message["self_id"]
-
-        # 连接响应启动定时任务
-        if self.auto_timing_start:
-            self.timing_start()
+        self.set_bot_status(message)
+        if self.messageSql is True:
+            self.cqapi._create_sql_link(self.messageSqlPath, self.messageSqlClearTime)
     
     def meta_event_connect(self, message):
         """
         连接响应
         """
-        if self.auto_timing_start:
-            self._meta_event_connect(message)
-            
+        self._meta_event_connect(message)
         logging.info("成功连接 websocket 服务! bot qq:%s" % self.__bot_qq)
     
-    def _timing_start(self):
+    def meta_event(self, message):
         """
-        定时任务
+        心跳
         """
-        def loop(job):
-            run_count = 0
-            while True:
-                self.timing_jobs_start(job, run_count)
-                for group_id in self.__group_id_list:
-                    if group_id in job["ban"]:
-                        return
-                    
-                    run_count += 1
-                    try:
-                        job["function"](group_id)
-                        self.timing_job_end(job, run_count, group_id)
-
-                    except Exception as err:
-                        self.runTimingError(job, run_count, err, group_id)
-
-                self.timing_jobs_end(job, run_count)
-                time.sleep(job["timeSleep"])
-        
-        for timing_job_key in self.timingList.keys():
-            
-            if "ban" not in self.timingList[timing_job_key]:
-                self.timingList[timing_job_key]["ban"] = []
-            
-            self.timingList[timing_job_key]["name"] = timing_job_key
-            thread = Thread(target=loop, args=(self.timingList[timing_job_key],), 
-                name=timing_job_key)
-            thread.setDaemon(True)
-            thread.start()
+        self.set_bot_status(message)
+        self._go_cqhttp_status = message["status"]
     
     def timing_start(self):
         """
@@ -361,7 +628,8 @@ class cqBot(cqSocket):
         """
         群列表定时任务执行完成
         """
-        logging.info("定时任务 %s 执行完成! 共执行 %s 次" % (job["name"], run_count))
+        logging.debug("定时任务 %s 执行完成! 共执行 %s 次" % (job["name"], run_count))
+        pass
     
     def runTimingError(self, job, run_count, err, group_id):
         """
@@ -369,18 +637,7 @@ class cqBot(cqSocket):
         """
         logging.error("定时任务 %s 在群 %s 执行错误... 共执行 %s 次 Error: %s" % (job["name"], group_id, run_count, err))
         logging.exception(err)
-    
-    def _set_help_text(self):
-        """
-        帮助信息模版
-        """
-        help_command_text = ""
-        for command in self.commandList.keys():
-            for help_r_text in self.commandList[command]["help"]:
-                help_command_text += help_r_text + "\n"
-        
-        self.help_text = self.help_text.format(help_command_text=help_command_text)
-    
+
     def user_log_srt(self, message):
         user_id = message["user_id"]
         if message["message_type"] == "private":
@@ -405,35 +662,17 @@ class cqBot(cqSocket):
         指令解析
         """
 
-        commandSign = list(message)[0]
+        if self.commandSign != "":
+            commandSign = list(message)[0]
+        else:
+            commandSign = ""
+
         command_str_list = message.split(" ")
         command = command_str_list[0].lstrip(commandSign)
         commandData = command_str_list[1:]
 
         return commandSign, command, commandData
     
-    def _check_set_command(self):
-        """
-        检查指令设置
-        """
-        for command in self.commandList.keys():
-            if "type" not in self.commandList[command]:
-                self.commandList[command]["type"] = "group"
-            
-            if "admin" not in self.commandList[command]:
-                self.commandList[command]["admin"] = False
-            
-            if "user" not in self.commandList[command]:
-                self.commandList[command]["user"] = ["all"]
-            else:
-                self.commandList[command]["user"] = self.commandList[command]["user"].split(",")
-            
-            if "ban" not in self.commandList[command]:
-                self.commandList[command]["ban"] = []
-            
-            if "help" not in self.commandList[command]:
-                self.commandList[command]["help"] = ""
-    
     def _check_command(self, message, command_type):
         """
         指令检查
@@ -443,26 +682,26 @@ class cqBot(cqSocket):
 
         if commandSign != self.commandSign:
             return False
-        
+
         if "group_id" in message:
             from_id = message["group_id"]
         else:
             from_id = message["user_id"]
-
-        if command not in self.commandList:
+        
+        if command not in self._commandList:
             self.notCommandError(message, from_id)
             return False
 
-        if self.commandList[command]["type"] != command_type and self.commandList[command]["type"] != "all":
+        if self._commandList[command]["type"] != command_type and self._commandList[command]["type"] != "all":
             return False
         
         self.check_command(message, from_id)
         
-        if from_id in self.commandList[command]["ban"]:
+        if from_id in self._commandList[command]["ban"]:
             self.banCommandError(message, from_id)
             return False
         
-        user_list = self.commandList[command]["user"]
+        user_list = self._commandList[command]["user"]
         if user_list[0] != "all" and message["message_type"] == "group":
             if "role" not in message["sender"] and "anonymous" not in user_list:
                 self.userPurviewError(message, from_id)
@@ -472,7 +711,7 @@ class cqBot(cqSocket):
                     self.userPurviewError(message, from_id)
                     return False
         
-        if self.commandList[command]["admin"] and message["user_id"] not in self.admin:
+        if self._commandList[command]["admin"] and message["user_id"] not in self.admin:
             self.purviewError(message, from_id)
             return False
 
@@ -482,23 +721,40 @@ class cqBot(cqSocket):
         """
         指令运行
         """
+        def run_command(message, cqCode_list, command_type):
+            try:
+                commandIn = self._check_command(message, command_type)
+                if not commandIn:
+                    return
 
+                commandSign, command, commandData, from_id = commandIn
+                self._commandList[command]["function"](commandData, cqCode_list, message, from_id)
+
+            except Exception as err:
+                self.runCommandError(message, err, from_id)
+        
+        thread = Thread(target=run_command, args=(message, cqCode_list, command_type, ), name="command")
+        thread.setDaemon(True)
+        thread.start()
+        
+    def _run_record_command(self, record, message, cqCode_list):
         try:
-            commandIn = self._check_command(message, command_type)
-            if not commandIn:
-                return
-
-            commandSign, command, commandData, from_id = commandIn
-            self.commandList[command]["function"](commandData, cqCode_list, message, from_id)
-
+            logging.info("%s 运行记录指令: %s: %s" % (self.user_log_srt(message), record["record_message"]["message"], message["message"]))
+            self._commandList[record["record_command"]]["backFunction"](message, record, cqCode_list)
         except Exception as err:
-            self.runCommandError(message, err, from_id)
+            self.runRecordCommandError(record, err)
     
     def check_command(self, message, from_id):
         """
         指令开始检查勾子
         """
         logging.info("%s 使用指令: %s" % (self.user_log_srt(message), message["message"]))
+
+    def on_group_msg(self, message, cqCode_list):
+        pass
+    
+    def on_private_msg(self, message, cqCode_list):
+        pass
     
     def _message(self, message):
         """
@@ -506,7 +762,12 @@ class cqBot(cqSocket):
         """
 
         # 解析 cqCode
-        return strToCqCodeToDict(message["message"])
+        cqCode_list = strToCqCodeToDict(message["message"])
+        # 检查等待回复
+        if self.cqapi._reply_ck(message["user_id"]):
+            self.cqapi._reply_add(message["user_id"], message)
+        
+        return cqCode_list
 
     def _message_private(self, message):
         """
@@ -516,11 +777,7 @@ class cqBot(cqSocket):
             return
 
         cqCode_list = self._message(message)
-        try:
-            self.on_private_msg(message, cqCode_list) 
-        except TypeError:
-            pass
-        
+        self.on_private_msg(message, cqCode_list)
         self._run_command(message, cqCode_list, "private")
     
     def _message_group(self, message):
@@ -531,11 +788,7 @@ class cqBot(cqSocket):
             return
 
         cqCode_list = self._message(message)
-        try:
-            self.on_group_msg(message, cqCode_list) 
-        except TypeError:
-            pass
-
+        self.on_group_msg(message, cqCode_list)
         for cqCode in cqCode_list:
             if cqCode["type"] == "at":
                 if cqCode["data"]["qq"] == str(self.__bot_qq):
@@ -596,6 +849,10 @@ class cqBot(cqSocket):
         """
         指令不存在时错误
         """
+        # commandSign 为空时不处理 不然每一条消息都会调用 notCommandError ...
+        if self.commandSign == "":
+            return
+
         self._bot_message_log("指令 %s 不存在..." % message["message"], message)
     
     def banCommandError(self, message, from_id):
