@@ -1,8 +1,9 @@
 import logging
 import json
-from pycqBot.cqCode import image
+from pycqBot.cqCode import image, node_list
 from lxml import etree
 import requests
+import random
 
 
 class Bilibili:
@@ -511,9 +512,9 @@ class Bilibili:
             # 记录的新动态被删除 (获取的新动态时间小干记录的新动态)
             if dynamic_list[uid]["time"] < self._dynamic_list_old[uid]["time"]:
                 # 如果是视频更新 time 字段无法正常判断, 重新检查
-                if dynamic["desc"]["type"] == 8:
+                if dynamic_list[uid]["data"]["desc"]["type"] == 8:
                     # 检查动态 id
-                    if dynamic_list[uid]["desc"]["dynamic_id"]  == self._dynamic_list_old[uid]["desc"]["dynamic_id"]:
+                    if dynamic_list[uid]["data"]["desc"]["dynamic_id"]  == self._dynamic_list_old[uid]["data"]["desc"]["dynamic_id"]:
                         continue
 
                 dynamic = self.set_dynamic_delete_message(await self._dynamic_check(
@@ -602,4 +603,280 @@ class Bilibili:
         """
         请求 bilibili api 时错误
         """
-        logging.error("请求 bilibili api发生错误! Error: %s code:%s" % (err_msg, code))
\ No newline at end of file
+        logging.error("请求 bilibili api发生错误! Error: %s code:%s" % (err_msg, code))
+
+
+class pixiv:
+
+    def __init__(self, cqapi, forward_name, forward_qq, proxy=None, cookie="", max_rlen=10, max_pid_len=20):
+        self.cqapi = cqapi
+        self._forward_qq = forward_qq
+        self._forward_name = forward_name
+        self._max_rlen = max_rlen
+        self._max_pid_len = max_pid_len
+        self._proxy= "http://%s" % proxy
+        self._headers = [
+            "referer=https://www.pixiv.net/",
+            "cookie=%s" % cookie
+        ]
+
+        self._pyheaders = {
+            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/98.0.4758.102 Safari/537.36",
+            "cookie": cookie
+        }
+    
+    def _json_data_check(self, data):
+        if data["error"]:
+            self.pixivApiError(data)
+            return False
+        
+        return data
+    
+    async def send_image_list(self, image_list, message):
+        """
+        转发图片表
+        """
+        message_id_list = []
+        for image_url in image_list:
+            cache_file = await self.cqapi._cqhttp_download_file(image_url, self._headers)
+            message_id_list.append(image("file://%s" % cache_file))
+
+        self.cqapi.send_group_forward_msg(message["group_id"], node_list(message_id_list, 
+            self._forward_name,
+            self._forward_qq
+        ))
+
+    async def search_image(self, search_data, page):
+        api = "https://www.pixiv.net/ajax/search/artworks/%s?word=%s&order=date_d&mode=all&p=%s&s_mode=s_tag_full&type=all&lang=zh" % (
+            search_data,
+            search_data,
+            page
+        )
+        return self._json_data_check(await self.cqapi.link(api, proxy=self._proxy, headers=self._pyheaders))
+    
+    async def user_image_id(self, user_id):
+        api = "https://www.pixiv.net/ajax/user/%s/profile/all?lang=zh" % (
+            user_id
+        )
+        return self._json_data_check(await self.cqapi.link(api, proxy=self._proxy, headers=self._pyheaders))
+    
+    async def get_image(self, img_id, message):
+        """
+        获取图片数据
+        """
+        try:
+            data = await self.cqapi.link("https://www.pixiv.net/ajax/illust/%s/pages?lang=zh" % img_id, proxy=self._proxy, headers=self._pyheaders)
+            self._json_data_check(data)
+            if data["error"]:
+                self.notImage(img_id, data["message"], message)
+                return False
+            
+            return data["body"]
+        except Exception as err:
+            self.getImageError(img_id, err)
+            return False
+
+    
+    async def get_user(self, user_name, nick=False):
+        """
+        获取用户
+        """
+        try:
+            # 完全匹配 (nick 啥意思啊?)
+            if nick:
+                nick = ""
+            else:
+                nick = "&nick_mf=1"
+
+            html_text = await self.cqapi.link("https://www.pixiv.net/search_user.php?s_mode=s_usr&nick=%s%s" % (user_name, nick), json=False, proxy=self._proxy, headers=self._pyheaders)
+
+            html = etree.HTML(html_text)
+            user_item = html.xpath('//li[@class="user-recommendation-item"]')
+            if len(user_item) == 0:
+                return False
+            
+            user_item = user_item[0]
+            user_id = user_item.xpath('./a/@href')[0].split("/")[-1]
+            user_count = user_item.xpath('./dl[@class="meta inline-list"]/dd//text()')
+            user_caption = user_item.xpath('./p[@class="caption"]//text()')
+            
+            user_item = {
+                "user_name": user_name,
+                "user_id": user_id,
+                "user_count": user_count,
+                "user_caption": user_caption
+            }
+
+        except Exception as err:
+            self.getUserError(user_name, nick, err)
+            return False
+
+        return user_item
+    
+    async def random_image(self, image_list, rlen, message):
+        random_image_list = []
+        img_id_list = []
+        # 随机原图
+        for _ in range(0, int(rlen)):
+            while True:
+                img_id = random.choice(image_list)
+
+                if img_id in img_id_list:
+                    continue
+
+                if "id" in img_id:
+                    img_id_list.append(img_id["id"])
+                else:
+                    img_id_list.append(img_id)
+
+                break
+
+            img_data = await self.get_image(img_id_list[-1], message)
+            if not img_data:
+                continue
+                
+            random_image_list.append(img_data[0]["urls"]["original"])
+        
+        return random_image_list
+    
+    async def _search_image_random(self, search_data, rlen, message):
+        """
+        随机搜索
+        """
+            
+        try:
+            if int(rlen) > self._max_rlen:
+                self.maxRlen(rlen, message)
+                rlen = self._max_rlen
+
+            # 获取数据量
+            data = await self.search_image(search_data, 1)
+            if not data:
+                return
+            
+            count = data["body"]["illustManga"]["total"]
+            if count == 0:
+                self.none_search_image(search_data, rlen, message)
+                return 
+            
+            page = int(count / 60)
+            if page > 1:
+                random_page = random.randint(1, page)
+            else:
+                random_page = 1
+
+            # 获取页数据
+            data = await self.search_image(search_data, random_page)
+            image_list = data["body"]["illustManga"]["data"]
+
+            if len(image_list) < int(rlen):
+                rlen = len(image_list)
+                self.insufficient_search_image(search_data, rlen, message)
+            
+            await self.send_image_list(await self.random_image(image_list, rlen, message), message)
+        except Exception as err:
+            self.randomSearchImageError(search_data, rlen, err)
+            return False
+    
+    async def _search_user_image_random(self, user_name, rlen, message, nick=False):
+        """
+        随机用户作品
+        """
+
+        try:
+            if int(rlen) > self._max_rlen:
+                self.maxRlen(rlen, message)
+                rlen = self._max_rlen
+
+            # 获取用户作品 id
+            user_item = await self.get_user(user_name, nick)
+            if not user_item:
+                self.searchNotUser(user_name, rlen, nick, message)
+                return False
+
+            user_image_id_list = await self.user_image_id(user_item["user_id"])
+            user_image_id_list = list(user_image_id_list["body"]["illusts"].keys())
+
+            await self.send_image_list(await self.random_image(user_image_id_list, rlen, message), message)
+        except Exception as err:
+            self.randomSearchUserImageError(user_name, rlen, nick, err)
+            return False
+    
+    async def _search_pid(self, pid, message):
+        try:
+            img_data = await self.get_image(pid, message)
+            if not img_data:
+                return False
+            
+            if len(img_data) > self._max_pid_len:
+                self.maxPidLen(len(img_data), message)
+                img_data = img_data[0: self._max_pid_len]
+
+            send_img = []
+            for img in img_data:
+                send_img.append(img["urls"]["original"])
+                
+            await self.send_image_list(send_img, message)
+        except Exception as err:
+            self.searchPidError(pid, err)
+            return False
+    
+    def search_image_random(self, search_data, rlen, message):
+        self.cqapi.add_task(self._search_image_random(search_data, rlen, message))
+    
+    def search_user_image_random(self, user_name, rlen, message, nick=False):
+        self.cqapi.add_task(self._search_user_image_random(user_name, rlen, message, nick))
+    
+    def search_pid(self, pid, message):
+        self.cqapi.add_task(self._search_pid(pid, message))
+    
+    def none_search_image(self, search_data, rlen, message):
+        self.cqapi.send_reply(message, "%s 没有搜索到数据..." % search_data)
+    
+    def insufficient_search_image(self, search_data, rlen, message):
+        self.cqapi.send_reply(message, "%s 数据量不足于指定量，将全部返回%s张" % (search_data, rlen))
+    
+    def notImage(self, img_id, err_msg, message):
+        self.cqapi.send_group_msg(message["group_id"], "无法获得到 %s，%s" % (img_id, err_msg))
+    
+    def searchNotUser(self, user_name, rlen, nick, message):
+        self.cqapi.send_group_msg(message["group_id"], "没有 %s 的用户" % user_name)
+    
+    def maxRlen(self, rlen, message):
+        self.cqapi.send_group_msg(message["group_id"], "指定量 %s 超出最大值 %s，将返回最大值" % (rlen, self._max_rlen))
+    
+    def maxPidLen(self, rlen, message):
+        self.cqapi.send_group_msg(message["group_id"], "指定量 %s 超出 PID 获取最大值 %s，将截取" % (rlen, self._max_pid_len))
+    
+        
+    def randomSearchImageError(self, search_data, rlen, err):
+        """
+        搜索随机图片时错误
+        """
+        logging.error("搜索 %s 随机图片时发生错误! Error: %s " % (search_data, err))
+        logging.exception(err)
+    
+    def randomSearchUserImageError(self, user_name, rlen, nick, err):
+        """
+        随机用户作品时错误
+        """
+        logging.error("随机 %s 作品时发生错误! Error: %s " % (user_name, err))
+        logging.exception(err)
+    
+    def searchPidError(self, pid_list, err):
+        logging.error("搜索 PID %s 时发生错误! Error: %s " % (pid_list, err))
+        logging.exception(err)
+    
+    def getImageError(self, img_id, err):
+        logging.error("搜索图片 %s 时发生错误! Error: %s " % (img_id, err))
+        logging.exception(err)
+    
+    def getUserError(self, user_name, nick, err):
+        logging.error("搜索用户 %s 时发生错误! Error: %s " % (user_name, err))
+        logging.exception(err)
+
+    def pixivApiError(self, err_msg):
+        """
+        请求 pixiv api 时错误
+        """
+        logging.error("请求 pixiv api发生错误! Error: %s " % err_msg)
\ No newline at end of file
